 clear all;

load('../data/blossom.mat');

z999 = 3.290526731492;% 99.9 sigma
Nrun = 78*16*4*2000; % Number of runs per data point
Nx = 21; % Number of physical error values per test round
x = logspace(log10(0.03),log10(0.3),Nx);
d = [3 5 7 9]; % Distance
q = [4 8 16 32 64 128 256]; % Number of fractional quantization levels
ls0 = [8 24 64 80];           % Number of inputs for the nn
ls1 = [4 8 16 32 64 128 256]; % Number of inputs for the first hidden layer
ls2 = [4 8 16 32 64];            % Number of inputs for the second hidden layer
tf = [1 3 2];
rot = [0 1];

f = figure;
c = get(gca,'colororder'); % plot colors
close;

expr_train = '\d+:\s+\d+.\d+\s+\d+.\d+\n';
expr_test = '';
expr_bits = '\d+\s+bits\n';
for ix = 1:Nx
    expr_test = [expr_test '\d+.\d+\s+\d+.\d+\n'];
end
expr_bits = [expr_bits expr_test];
expr_test = ['\n' expr_test];

%% tf rot data
data_tf_train = zeros(size(d,2),size(ls1,2),size(ls2,2),2,2,75);
data_tf_test = zeros(size(d,2),size(ls1,2),size(ls2,2),2,2,Nx);
data_tf_test_pth = zeros(size(d,2),size(ls1,2),size(ls2,2),2,2);
data_tf_test_pth_ds = zeros(size(d,2),size(ls1,2),size(ls2,2),2,2);

for id = 1:size(d,2)
    for i1 = 1:size(ls1,2)
        for i2 = 1:size(ls2,2)
            for itf = 1:size(tf,2)
                for irot = 1:size(rot,2) % rotational weight symmetry
                    
                    % if file exists
                    fid = fopen(['../data/training_data/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);
                    if(fid == -1)
                        data_tf_train(id,i1,i2,itf,irot,:) = zeros(1,size(data_tf_train,6));
                        data_tf_test(id,i1,i2,itf,irot,:) = zeros(1,Nx);
                        data_tf_test_pth(id,i1,i2,itf,irot) = 0;
                        data_tf_test_pth_ds(id,i1,i2,itf,irot) = 0;
                        disp(['Could not find file: ../data/training_data/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);
                    else
                        fclose(fid);
                        % load data
                        filetext = fileread(['../data/training_data/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);
                        
                        match_train = split(regexp(filetext, expr_train , 'match'));
                        match_test = split(regexp(filetext, expr_test, 'match'));
                        
                        data_tf_train(id,i1,i2,itf,irot,:) = str2double(match_train(1,end-size(data_tf_train,6)+1:end,3)');
                        data_tf_test(id,i1,i2,itf,irot,:) = str2double(match_test(3:2:(2*Nx+1),1));
                        
                        % calculate pseudo threshold test
                        data_tf_test_pth(id,i1,i2,itf,irot) = 0;
                        data_tf_test_pth_ds(id,i1,i2,itf,irot) = 0;
                        s = 1;
                        for ix = 1:Nx
                            dy1 = data_tf_test(id,i1,i2,itf,irot,ix);
                            if(s ~= sign(x(ix)-dy1))
                                s = sign(x(ix)-dy1);
                                if(ix > 1)
                                    dy2 = data_tf_test(id,i1,i2,itf,irot,ix-1);
                                    a = (log(dy1/dy2))/(log(x(ix)/x(ix-1)));
                                    data_tf_test_pth(id,i1,i2,itf,irot) = exp((log(dy1) - a * log(x(ix)))/(1 - a));
                                    data_tf_test_pth_ds(id,i1,i2,itf,irot) = sqrt( (dy2.*(1-dy2) + dy1.*(1-dy1)) / Nrun );
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end

%% quantization data
data_quant_train = zeros(size(d,2),size(q,2),size(ls1,2),size(ls2,2),50);
data_quant_test = zeros(size(d,2),size(q,2),size(ls1,2),size(ls2,2),Nx);
data_quant_bits = zeros(size(d,2),size(q,2),size(ls1,2),size(ls2,2),7,Nx);

data_quant_test_pth = zeros(size(d,2),size(q,2),size(ls1,2),size(ls2,2));
data_quant_bits_pth = zeros(size(d,2),size(q,2),size(ls1,2),size(ls2,2),7);
data_quant_test_pth_ds = zeros(size(d,2),size(q,2),size(ls1,2),size(ls2,2));
data_quant_bits_pth_ds = zeros(size(d,2),size(q,2),size(ls1,2),size(ls2,2),7);

for id = 1:size(d,2)
    % calculate pseudo threshold blossom
    data_blossom_pth(id) = 0;
    data_blossom_pth_ds(id) = 0;
    s = 1;
    for ix = 1:Nx
        dy1 = bl(id,ix);
        if(s ~= sign(x(ix)-dy1))
            s = sign(x(ix)-dy1);
            if(ix > 1)
                dy2 = bl(id,ix-1);
                a = (log(dy1/dy2))/(log(x(ix)/x(ix-1)));
                data_blossom_pth(id) = exp((log(dy1) - a * log(x(ix)))/(1 - a));
                data_blossom_pth_ds(id) = sqrt( (dy2.*(1-dy2) + dy1.*(1-dy1)) / it(ix));
            end
        end
    end
end

for id = 1:size(d,2)
    for iq = 1:size(q,2)
        for i1 = 1:size(ls1,2)
            for i2 = 1:size(ls2,2)
                itf = 3;  % sqnl
                irot = 1; % rotational weight symmetry
                
                % if file exists
                fid = fopen(['../data/training_data/d' num2str(d(id)) '_q' num2str(q(iq)) '_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(itf) '_r' num2str(irot)]);
                if(fid == -1)
                    data_quant_train(id,iq,i1,i2,:) = zeros(1,size(data_quant_train,5));
                    data_quant_test(id,iq,i1,i2,:) = zeros(1,Nx);
                    data_quant_test_pth(id,iq,i1,i2) = 0;
                    data_quant_test_pth_ds(id,iq,i1,i2) = 0;
                    for ib = 1:size(bits,2)
                        data_quant_bits_pth(id,iq,i1,i2,ib) = 0;
                        data_quant_bits_pth_ds(id,iq,i1,i2,ib) = 0;
                        data_quant_bits(id,iq,i1,i2,ib,:) = zeros(1,Nx);
                    end
                    disp(['Could not find file: ../data/training_data/d' num2str(d(id)) '_q' num2str(q(iq)) '_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(itf) '_r' num2str(irot)]);
                else
                    fclose(fid);
                    % load data
                    filetext = fileread(['../data/training_data/d' num2str(d(id)) '_q' num2str(q(iq)) '_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(itf) '_r' num2str(irot)]);
                    
                    match_train = split(regexp(filetext, expr_train , 'match'));
                    match_test = split(regexp(filetext, expr_test, 'match'));
                    match_bits = split(regexp(filetext, expr_bits, 'match'));
                    
                    data_quant_train(id,iq,i1,i2,:) = str2double(match_train(1,1:50,3)');
                    data_quant_test(id,iq,i1,i2,:) = str2double(match_test(1,1,3:2:(2*Nx+1)));
                    bits = str2double(match_bits(1,:,1));
                    for ib = 1:size(bits,2)
                        data_quant_bits(id,iq,i1,i2,ib,:) = str2double(match_bits(1,ib,4:2:(2+2*Nx)));
                    end
                    
                    % calculate pseudo threshold test
                    data_quant_test_pth(id,iq,i1,i2) = 0;
                    data_quant_test_pth_ds(id,iq,i1,i2) = 0;
                    s = 1;
                    for ix = 1:Nx
                        dy1 = data_quant_test(id,iq,i1,i2,ix);
                        if(s ~= sign(x(ix)-dy1))
                            s = sign(x(ix)-dy1);
                            if(ix > 1)
                                dy2 = data_quant_test(id,iq,i1,i2,ix-1);
                                a = (log(dy1/dy2))/(log(x(ix)/x(ix-1)));
                                data_quant_test_pth(id,iq,i1,i2) = exp((log(dy1) - a * log(x(ix)))/(1 - a));
                                data_quant_test_pth_ds(id,iq,i1,i2) = sqrt( (dy2.*(1-dy2) + dy1.*(1-dy1)) / Nrun );
                            end
                        end
                    end
                    
                    % calculate pseudo threshold bits
                    for ib = 1:size(bits,2)
                        data_quant_bits_pth(id,iq,i1,i2,ib) = 0;
                        data_quant_bits_pth_ds(id,iq,i1,i2,ib) = 0;
                        s = 1;
                        for ix = 1:Nx
                            dy1 = data_quant_bits(id,iq,i1,i2,ib,ix);
                            if(s ~= sign(x(ix)-dy1))
                                s = sign(x(ix)-dy1);
                                if(ix > 1)
                                    dy2 = data_quant_bits(id,iq,i1,i2,ib,ix-1);
                                    a = (log(dy1/dy2))/(log(x(ix)/x(ix-1)));
                                    data_quant_bits_pth(id,iq,i1,i2,ib) = exp((log(dy1) - a * log(x(ix)))/(1 - a));
                                    data_quant_bits_pth_ds(id,iq,i1,i2,ib) = sqrt( (dy2.*(1-dy2) + dy1.*(1-dy1)) / Nrun );
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end

save('raw_data.mat', 'ls0', 'ls1', 'ls2', 'x', 'Nrun', 'Nx', 'q', 'bits', 'd', 'c', 'z999', 'tf', 'rot', ...
    'data_tf_train', 'data_tf_test', 'data_tf_test_pth', 'data_tf_test_pth_ds', ...
    'data_quant_bits','data_quant_bits_pth','data_quant_bits_pth_ds', ...
    'data_quant_train', ...
    'data_quant_test','data_quant_test_pth','data_quant_test_pth_ds', ...
    'data_blossom_pth','data_blossom_pth_ds');