clear all; close all;


d = [3 5 7 9];
NNin = [8 24 48 80];

load('../data/data_hardware/data_node_area.mat');
load('raw_data.mat');
load('../data/blossom.mat');
load('slope_data.mat');

data_area_log = data_area_log/1e6;

area_max = [7e-3 1e-2 3e-2 1e-1 3e-1 1 3 10];
marker = ['p' 'v' 'h' '^' 'o' '>' 's' 'd'];

%% calculate all areas
area = zeros(4,7,7,5,7);
for dd = 1:numel(d)
    for ii = 2:8
        for jj = 2:6
            for b = 3:9
                for q = 2:8
                    area(dd,q-1,ii-1,jj-1,b-2) = 2^ii * data_area_log(NNin(dd),b,1,b) + ...
                                                2^jj * data_area_log(2^ii,b,b,b) + ...
                                                2 * data_area_log(2^jj,b,b,1);
                end
            end
        end
    end
end

area_re = reshape(area,size(d,2),numel(area)/size(d,2),1);
log_re = reshape(data_quant_bits,size(d,2),numel(area)/size(d,2),Nx);
log_re_z = log_re > 0;

%%
f = figure;
loglog(x,x,'--k');
hold on;
min_log_d = zeros(1,4);
min_log = zeros(1,Nx);
cd = zeros(Nx,3);
% for every plot
for ia = 1:size(area_max,2)
    ii_a = area_re < area_max(ia);
    % loop through all physical error rates
    for ip = 1:Nx
        % find minimum logical error for every distance
        for id = 1:size(d,2)
           min_log_d(id) = min(log_re(id,log_re_z(id,:,ip) & ii_a(id,:),ip));
        end
        [v, id] = min(min_log_d);
        min_log(ip) = v;
        cd(ip,:) = c(id,:);
    end
    loglog(x,min_log,[marker(ia) '-k']);
    for ip = 1:Nx
       loglog(x(ip),min_log(ip),marker(ia),'MarkerEdgeColor','k','MarkerFaceColor',cd(ip,:),'HandleVisibility','off'); 
    end
end
legend('y = x',...
    ['area < ' num2str(area_max(1)) ' mm^2'],...
    ['area < ' num2str(area_max(2)) ' mm^2'],...
    ['area < ' num2str(area_max(3)) ' mm^2'],...
    ['area < ' num2str(area_max(4)) ' mm^2'],...
    ['area < ' num2str(area_max(5)) ' mm^2'],...
    ['area < ' num2str(area_max(6)) ' mm^2'],...
    ['area < ' num2str(area_max(7)) ' mm^2'],...
    'best ( < 10 mm^2 )',...
    'Location','SouthEast');

xlim([0.03 0.2]);
xlabel('Physical Error Rate');
ylabel('Logical Error Rate');
title('Optimal Logical Error vs Area');
grid on;

f.PaperType = 'a5';
f.PaperSize = f.PaperSize ./ [1 1.8];
f.PaperOrientation = 'portrait';
print(['../plots/pareto/logic_vs_area'],'-dpdf','-fillpage');