clear all;
load('raw_data.mat');
load('../data/blossom.mat');
close all;

%% blossom slope
f=figure(20);
loglog(x,x,'k--');
hold on;
for id = 1:size(d,2)
    errorbar(x,bl(id,:),z999.*sqrt(bl(id,:).*(1-bl(id,:))./it),'.','Color',c(id,:));
    ff = fit(x',bl(id,:)','a*(x/a)^(b*(1-c*(x)))','Start',[data_blossom_pth(id), ((d(id)+1)/2), 1],'Exclude',x>0.18,'Lower',[0.01, (d(id)-1)/2, 0.01],'Upper',[0.2 (d(id)+2)/2,100],'Weights',[100 100 100 100 ones(1,17)]);
    blossom_fit(id,:) = [ff.a ff.b];
    loglog(x,ff.a .* (x ./ ff.a).^(ff.b*(1-ff.c.*x)),'-','Color',c(id,:));
    %loglog(x,ff.a .* (x ./ ff.a).^(ff.b),'Color',c(id,:));
end
hold off;
title('Model Fit on MWPM Data');
xlabel('Physical Error Rate');
ylabel('Logical Error Rate');
legend('y = x','Distance 3 Data','Distance 3 Model','Distance 5 Data','Distance 5 Model','Distance 7 Data','Distance 7 Model','Distance 9 Data','Distance 9 Model','Location','SouthEast');

f.PaperType = 'a5';
f.PaperOrientation = 'portrait';
f.PaperSize = f.PaperSize ./ [1 1.8];
%print('../plots/raw/blossomslope','-dpdf');

%% plot all quantization curves
figure(1);
for s = 1:4
    subplot(2,2,s);loglog(x,x,'k--');hold on;
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
end
for id = 1:size(d,2)
    for iq = 1:size(q,2)
        for i1 = 1:size(ls1,2)
            for i2 = 1:size(ls2,2)
                subplot(2,2,id);
                loglog(x,squeeze(data_quant_test(id,iq,i1,i2,:)),'Color',c(iq,:));
                loglog(squeeze(data_quant_test_pth(id,iq,i1,i2)),squeeze(data_quant_test_pth(id,iq,i1,i2)),'x','Color',c(iq,:));
            end
        end
    end
end
for s = 1:4
    subplot(2,2,s);
    loglog(x,x,'k--');
    loglog(xbl,bl(s,:),'r-.');
    loglog(data_blossom_pth(s),data_blossom_pth(s),'rx');
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
    ylim([4e-4,1e-0]);
end

figure(2);
x = logspace(log10(0.03),log10(0.3),Nx);
for s = 1:4
    subplot(2,2,s);loglog(x,x,'k--');hold on;
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
end
for id = 1:size(d,2)
    for iq = 1:size(q,2)
        for i1 = 1:size(ls1,2)
            for i2 = 1:size(ls2,2)
                for ib = 1:size(bits,2)
                    subplot(2,2,id);
                    loglog(x,squeeze(data_quant_bits(id,iq,i1,i2,ib,:)),'Color',c(ib,:));
                    loglog(squeeze(data_quant_bits_pth(id,iq,i1,i2,ib)),squeeze(data_quant_bits_pth(id,iq,i1,i2,ib)),'x','Color',c(iq,:));
                end
            end
        end
    end
end
for s = 1:4
    subplot(2,2,s);
    loglog(x,x,'k--');
    loglog(xbl,bl(s,:),'r-.');
    loglog(data_blossom_pth(s),data_blossom_pth(s),'rx');
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
    ylim([4e-4,1e-0]);
end

%% plot point at x = 0.03 vs pth
f = figure(3);
for id = 1:size(d,2)
    subplot(2,2,id);
    datax1 = reshape(data_quant_bits(id,:,:,:,:,1),[],1,1,1,1,1);
    datay1 = reshape(data_quant_bits_pth(id,:,:,:,:),[],1,1,1,1);
    loglog(datax1,datay1,'xb');
    hold on;
    datax2 = reshape(data_quant_test(id,:,:,:,1),[],1,1,1,1);
    datay2 = reshape(data_quant_test_pth(id,:,:,:),[],1,1,1);
    loglog(datax2,datay2,'xk');
    loglog(bl(id,1),data_blossom_pth(id),'rx');
    
    datax = [bl(id,1);datax1;datax2];
    datay = [data_blossom_pth(id);datay1;datay2];
    
    f1 = fit(datax,datay,'a*(x/b)^c','Start',[data_blossom_pth(id) bl(id,1) (-(2/d(id)))],'Exclude',datay<(0.6*(max(datay))),'Lower',[0.01,0.01,-1],'Upper',[0.5,0.5,-0.01]);
    plot(f1);
    
    xlabel('LER at PER = 0.03');
    ylabel('Pth');
    title(['Distance ' num2str(d(id))]);
    grid on;
end
f.PaperType = 'a5';
f.PaperOrientation = 'portrait';
%print('../plots/raw/pth_vs_ler','-dpdf');


clear data_quant_test_slope data_quant_bits_slope mindy maxdy mindx maxdx
%% plot point at pth vs slope at x = 0.03
f = figure(4);
for id = 1:size(d,2)    
    subplot(2,2,id);
    clear datax datay datax1 datax2 datay1 datay2 datax3 datay3
    for iq = 1:size(q,2)
        for i1 = 1:size(ls1,2)
            for i2 = 1:size(ls2,2)
                y = squeeze(data_quant_test(id,iq,i1,i2,:));
                pth = squeeze(data_quant_test_pth(id,iq,i1,i2));
                if(pth > x(2))
                    ff = fit(x',y,'a*(x/a)^(b*(1-c*(x)))','Start',[data_blossom_pth(id), ((d(id)+1)/2), 1],'Exclude',x>0.18,'Lower',[0.01, 1, 0.01],'Upper',[0.2 d(id),100],'Weights',[100 100 100 100 ones(1,17)]);
                    datay1(id,iq,i1,i2) = ff.b;
                    datax1(id,iq,i1,i2) = pth;
                else
                    datay1(id,iq,i1,i2) = 0;
                    datax1(id,iq,i1,i2) = 0;
                end
                
                for ib = 1:size(bits,2)
                    y = squeeze(data_quant_bits(id,iq,i1,i2,ib,:));
                    pth = squeeze(data_quant_bits_pth(id,iq,i1,i2,ib));
                    if(pth > x(2))
                        ff = fit(x',y,'a*(x/a)^(b*(1-c*(x)))','Start',[data_blossom_pth(id), ((d(id)+1)/2), 1],'Exclude',x>0.18,'Lower',[0.01, 1, 0.01],'Upper',[0.2 d(id),100],'Weights',[100 100 100 100 ones(1,17)]);
                        datay2(id,iq,i1,i2,ib) = ff.b;
                        datax2(id,iq,i1,i2,ib) = pth;
                    else
                        datay2(id,iq,i1,i2,ib) = 0;
                        datax2(id,iq,i1,i2,ib) = 0;
                    end
                end
            end
        end
    end
    
    for i1 = 1:size(ls1,2)
        for i2 = 1:size(ls2,2)
            for itf = 1:size(tf,2)
                for irot = 1:size(rot,2)
                    y = squeeze(data_tf_test(id,i1,i2,itf,irot,:));
                    pth = squeeze(data_tf_test_pth(id,i1,i2,itf,irot));
                    if(pth > x(2))
                        ff = fit(x',y,'a*(x/a)^(b*(1-c*(x)))','Start',[data_blossom_pth(id), ((d(id)+1)/2), 1],'Exclude',x>0.18,'Lower',[0.01, 1, 0.01],'Upper',[0.2 d(id),100],'Weights',[100 100 100 100 ones(1,17)]);
                        datay3(id,i1,i2,itf,irot) = ff.b;
                        datax3(id,i1,i2,itf,irot) = pth;
                    else
                        datay3(id,i1,i2,itf,irot) = 0;
                        datax3(id,i1,i2,itf,irot) = 0;
                    end
                end
            end
        end
    end
     
    dy1 = reshape(datay1,[],1,1,1);
    dx1 = reshape(datax1,[],1,1,1);
    dy2 = reshape(datay2,[],1,1,1,1);
    dx2 = reshape(datax2,[],1,1,1,1);
    dy3 = reshape(datay3,[],1,1,1,1);
    dx3 = reshape(datax3,[],1,1,1,1);
    
    dy(id) = {[dy1;dy2;dy3]};
    dx(id) = {[dx1;dx2;dx3]};
        
    if(id > 1)
        mindy(id) = min([dy1(dy1>0);dy2(dy2>0);dy3(dy3>0);blossom_fit(id-1,2)]);
    else
        mindy(id) = min([dy1(dy1>0);dy2(dy2>0);dy3(dy3>0);1]);
    end
    maxdy(id) = max([blossom_fit(id,2);dy1;dy2;dy3]);
    mindx(id) = min([dx1(dx1>0);dx2(dx2>0);dx3(dx3>0)]);
    maxdx(id) = max([dx1;dx2;dx3;data_blossom_pth(id)]);
    
    if(id > 1)
        semilogx(dx2,dy2,'.',dx1,dy1,'.',dx3,dy3,'.', ...
               [mindx(id), max(maxdx)]         , blossom_fit(id,2)   .* [1 1], 'k', ...
               [mindx(id), max(maxdx)]         , blossom_fit(id-1,2) .* [1 1], 'k--', ...
               [mindx(id), max(maxdx)]         , maxdy(id-1)         .* [1 1], 'r', ...
               data_blossom_pth(id)   .* [1 1] , [mindy(id), maxdy(id)], 'k', ...
               data_blossom_pth(id-1) .* [1 1] , [mindy(id), maxdy(id)], 'k--', ...
               maxdx(id-1)            .* [1 1] , [mindy(id), maxdy(id)], 'r');
    else
        semilogx(dx2,dy2,'.',dx1,dy1,'.', dx3,dy3,'.', ...
               [mindx(id), maxdx(id)]          , blossom_fit(id,2) .* [1 1], 'k', ...
               [mindx(id), maxdx(id)]          , [1 1], 'r', ...
               data_blossom_pth(id) .* [1 1]   , [mindy(id), maxdy(id)], 'k');        
    end
    
    ylim([mindy(id),maxdy(id)]);
    xlim([mindx(id),max(maxdx(1:id))]);
    
    ylabel('Slope at PER = 0.03)');
    xlabel('Pth');
    title(['Distance ' num2str(d(id))]);
    grid on;
    
    f.PaperType = 'a5';
    f.PaperSize = f.PaperSize ./ [1 1.8];
    f.PaperOrientation = 'portrait';
    %print(['../plots/raw/slopes_vs_pth'],'-dpdf','-fillpage');
    
    data_quant_bits_slope(id,:,:,:,:) = datay2(id,:,:,:,:);
    data_quant_test_slope(id,:,:,:) = datay1(id,:,:,:);
    data_tf_test_slope(id,:,:,:,:) = datay3(id,:,:,:,:);
end

blossom_slope = blossom_fit(:,2);
blossom_slope_th(1) = 1/blossom_fit(1,2);
blossom_slope_th(2:4) = blossom_fit(1:3,2) ./ blossom_fit(2:4,2);

%%
f = figure(40);
for id = 1:4
    dx1 = cell2mat(dx(id));
    dy1 = cell2mat(dy(id));
    %if(id > 1)
        semilogx(dx1(:),dy1(:),'.','Color',c(id,:));
        hold on;
        semilogx([min(mindx), max(maxdx)]         , blossom_fit(id,2)   .* [1 1], '--','Color',c(id,:),'HandleVisibility','off'); 
        %semilogx([min(mindx), max(maxdx)]         , maxdy(id)           .* [1 1], 'Color',c(id,:)); 
        semilogx(data_blossom_pth(id)    .* [1 1] , [min(mindy), 5], '--','Color',c(id,:),'HandleVisibility','off'); 
        %semilogx(maxdx(id)               .* [1 1] , [min(mindy), max(maxdy)], 'Color',c(id,:)); 
%     else
%         semilogx(dx1(:),dy1(:),'.','Color',c(id,:));
%         hold on;
%         semilogx( ...
%                [mindx(id), maxdx(id)]          , blossom_fit(id,2) .* [1 1], 'k', ...
%                [mindx(id), maxdx(id)]          , [1 1], 'r', ...
%                data_blossom_pth(id) .* [1 1]   , [mindy(id), maxdy(id)], 'k');        
%     end
    
end
ylabel('Slope');
xlabel('p_{th}');
title(['Slope vs p_{th}']);
grid on;
hold off;
f.PaperType = 'a5';
f.PaperSize = f.PaperSize ./ [1 1.8];
f.PaperOrientation = 'portrait';
legend('Distance 3','Distance 5','Distance 7','Distance 9','Location','NorthWest');
%print(['../plots/raw/slope_vs_pth'],'-dpdf','-fillpage');

%% plot all quantization curves for pth > 0.07
figure(5);
for s = 1:4
    subplot(2,2,s);loglog(x,x,'k--');hold on;
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
end
for id = 1:size(d,2)
    for iq = 1:size(q,2)
        for i1 = 1:size(ls1,2)
            for i2 = 1:size(ls2,2)
                subplot(2,2,id);
                if(data_quant_test_slope(id,iq,i1,i2) > blossom_slope_th(id))
                    loglog(x,squeeze(data_quant_test(id,iq,i1,i2,:)),'Color',c(iq,:));
                    loglog(squeeze(data_quant_test_pth(id,iq,i1,i2)),squeeze(data_quant_test_pth(id,iq,i1,i2)),'x','Color',c(iq,:));
                end
            end
        end
    end
end
for s = 1:4
    subplot(2,2,s);
    loglog(x,x,'k--');
    loglog(xbl,bl(s,:),'r-.');
    loglog(data_blossom_pth(s),data_blossom_pth(s),'rx');
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
    ylim([4e-4,1e-0]);
end

figure(6);
x = logspace(log10(0.03),log10(0.3),Nx);
for s = 1:4
    subplot(2,2,s);loglog(x,x,'k--');hold on;
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
end
for id = 1:size(d,2)
    for iq = 1:size(q,2)
        for i1 = 1:size(ls1,2)
            for i2 = 1:size(ls2,2)
                for ib = 1:size(bits,2)
                    subplot(2,2,id);
                    if(data_quant_bits_slope(id,iq,i1,i2,ib) > blossom_slope_th(id))
                        loglog(x,squeeze(data_quant_bits(id,iq,i1,i2,ib,:)),'Color',c(ib,:));
                        loglog(squeeze(data_quant_bits_pth(id,iq,i1,i2,ib)),squeeze(data_quant_bits_pth(id,iq,i1,i2,ib)),'x','Color',c(iq,:));
                    end
                end
            end
        end
    end
end
for s = 1:4
    subplot(2,2,s);
    loglog(x,x,'k--');
    loglog(xbl,bl(s,:),'r-.');
    loglog(data_blossom_pth(s),data_blossom_pth(s),'rx');
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
    ylim([4e-4,1e-0]);
end


%% plot all quantization curves
figure;
for s = 1:4
    subplot(2,2,s);loglog(x,x,'k--');hold on;
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
end
for id = 1:size(d,2)
    for i1 = 1:size(ls1,2)
        for i2 = 1:size(ls2,2)
            for itf = 1:size(tf,2)
                for irot = 1:size(rot,2)
                    subplot(2,2,id);
                    loglog(x,squeeze(data_tf_test(id,i1,i2,itf,irot,:)),'Color',c(i1,:));
                    loglog(squeeze(data_tf_test_pth(id,i1,i2,itf,irot)),squeeze(data_tf_test_pth(id,i1,i2,itf,irot)),'x','Color',c(i1,:));
                end
            end
        end
    end
end
for s = 1:4
    subplot(2,2,s);
    loglog(x,x,'k--');
    loglog(xbl,bl(s,:),'r-.');
    loglog(data_blossom_pth(s),data_blossom_pth(s),'rx');
    xlabel('Physical Error Rate');
    ylabel('Logical Error Rate');
    title(['Distance ' num2str(d(s))]);
    grid on;
    ylim([4e-4,1e-0]);
end

save('slope_data.mat','data_quant_bits_slope','data_quant_test_slope','data_tf_test_slope','blossom_slope');