clear all;

load('../data/blossom.mat');

z999 = 3.290526731492;% 99.9 sigma
Nrun = 78*16*4*2000; % Number of runs per data point
Nx = 21; % Number of physical error values per test round
x = logspace(log10(0.03),log10(0.3),Nx);
d = [3 5 7 9]; % Distance
q = [4 8 16 32 64 128 256]; % Number of fractional quantization levels
ls0 = [8 24 64 80];           % Number of inputs for the nn
ls1 = [4 8 16 32 64 128 256]; % Number of inputs for the first hidden layer
ls2 = [4 8 16 32 64];            % Number of inputs for the second hidden layer
tf = [1 3 2];
rot = [0 1];

f = figure;
c = get(gca,'colororder'); % plot colors
close;

expr_train = '\d+:\s+\d+.\d+\s+\d+.\d+\n';

data_tf_train = zeros(3,2,150);

for id = 4
    for i1 = 7
        for i2 = 5
            for itf = 1:size(tf,2)
                for irot = 1:size(rot,2) % rotational weight symmetry
                    
                    % if previous file exists
                    fid = fopen(['../cuda/data2/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);
                    if(fid == -1)
                        data_tf_train(itf,irot,:) = zeros(1,size(data_tf_train,3));
                        disp(['Could not find file: ../cuda/data2/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);
                        disp(['Looking for longer new file in ../data/']);
                    else
                        fclose(fid);
                        % load data
                        filetext = fileread(['../cuda/data2/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);

                        match_train = split(regexp(filetext, expr_train , 'match'));

                        data_tf_train(itf,irot,1:size(match_train,2)) = str2double(match_train(1,:,3)');
                    end
                        
                    % if file exists
                    fid = fopen(['../data/training_data/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);
                    if(fid == -1)
                        data_tf_train(itf,irot,:) = zeros(1,size(data_tf_train,3));
                        disp(['Could not find file: ../data/training_data/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);
                    else
                        fclose(fid);
                        % load data
                        filetext = fileread(['../data/training_data/d' num2str(d(id)) '_q0_' num2str(ls1(i1)) '_' num2str(ls2(i2)) '_t' num2str(tf(itf)) '_r' num2str(rot(irot))]);

                        match_train = split(regexp(filetext, expr_train , 'match'));

                        data_tf_train(itf,irot,151-size(match_train,2):150) = str2double(match_train(1,:,3)');

                    end
                end
            end
        end
    end
end

f = figure;
semilogy([1 150],[0.119319997728 0.119319997728],'k--');
hold on;

for itf = 1:size(tf,2)
    for irot = 1:size(rot,2) % rotational weight symmetry
        y = squeeze(data_tf_train(itf,irot,:));
        errorbar(1:150,y,z999.*sqrt(y.*(1-y)./Nrun),'Color',c((itf-1)*2+irot,:));
    end
end

for itf = 1:size(tf,2)
    for irot = 1:size(rot,2) % rotational weight symmetry
        semilogy([1 150],mean(squeeze(data_tf_train(itf,irot,140:150)))*[1 1],'--','Color',c((itf-1)*2+irot,:));
    end
end
legend('Blossom','TanH Unrot','TanH Rot','SQNL Unrot','SQNL Rot','ReLU Unrot','ReLU Rot');
xlabel('Training Iterations');
ylabel('Training Performance');
title({'Training Performance vs Iterations';'Distance 9, Shape : 80, 256, 64, 2'});

f.PaperType = 'a5';
f.PaperSize = f.PaperSize ./ [1 1.8];
f.PaperOrientation = 'portrait';
%print(['../plots/raw/training_d' num2str(d(id))],'-dpdf','-fillpage');

%%

f = figure;
semilogy([100 150],[0.119319997728 0.119319997728],'k--');
hold on;

for itf = 1:size(tf,2)
    for irot = 1:size(rot,2) % rotational weight symmetry
        y = squeeze(data_tf_train(itf,irot,100:150));
        errorbar(100:150,y,z999.*sqrt(y.*(1-y)./Nrun),'Color',c((itf-1)*2+irot,:));
    end
end

for itf = 1:size(tf,2)
    for irot = 1:size(rot,2) % rotational weight symmetry
        semilogy([100 150],mean(squeeze(data_tf_train(itf,irot,140:150)))*[1 1],'--','Color',c((itf-1)*2+irot,:));
    end
end
ylim([0.107 0.15]);
%legend('Blossom','TanH 0','TanH 1','SQNL 0','SQNL 1');
xlabel('Training Iterations');
ylabel('Training Performance');
%title({'Training Performance vs Iterations';'Distance 9, Shape : 80, 256, 64, 2'});

f.PaperType = 'a5';
f.PaperSize = f.PaperSize ./ [2 3.6];
f.PaperOrientation = 'portrait';
%print(['../plots/raw/training_d' num2str(d(id)) 'zoom'],'-dpdf','-fillpage');