#pragma once

#include "settings.h"
#include "functions.h"
#include "kernels.h"

class SC
{
	public:

	// DATA
	unsigned char *h_ancillas, *d_ancillas;
	unsigned char *h_targets, *d_targets;
	
	curandStateMRG32k3a *d_randstates;

	// Functions
	SC()
	{
		h_ancillas = (unsigned char*)checkMalloc(malloc(sizeof(unsigned char)*(DISTANCE*DISTANCE)*BATCH_SIZE));
		checkCudaError(cudaMalloc(&d_ancillas,sizeof(unsigned char)*(DISTANCE*DISTANCE)*BATCH_SIZE));
		h_targets = (unsigned char*)checkMalloc(malloc(sizeof(unsigned char)*BATCH_SIZE));
		checkCudaError(cudaMalloc(&d_targets,sizeof(unsigned char)*BATCH_SIZE));
	
		checkCudaError(cudaMalloc(&d_randstates,sizeof(curandStateMRG32k3a)*GRID_SIZE));

		init_randstates();

	}

	~SC()
	{
		free(h_ancillas);
		free(h_targets);

		cudaFree(d_ancillas);
		cudaFree(d_targets);

		cudaFree(d_randstates);
	}

	void init_randstates()
	{
		init_rng<<<N_BLOCK_PER_MP*N_MP,BLOCK_SIZE>>>(d_randstates);
		checkCudaError(cudaGetLastError());
	}

	void run_cycle()
	{
		generate_surface_code_data<<<N_BLOCK_PER_MP*N_MP,BLOCK_SIZE>>>(d_ancillas,d_targets,d_randstates);
		cudaDeviceSynchronize();
		checkCudaError(cudaGetLastError());
		checkCudaError(cudaMemcpy(h_targets,d_targets,sizeof(unsigned char)*BATCH_SIZE,cudaMemcpyDeviceToHost));
		cudaDeviceSynchronize();
	}

};
